#include <Wt/WHBoxLayout.h>
#include <Wt/WVBoxLayout.h>

#include "InteractiveContainer.h"

namespace bsw {
namespace halfagame {

    InteractiveContainer::InteractiveContainer()
    {
        rearrange(ratio_);
    }

    InteractiveContainer::~InteractiveContainer()
    {
    }

    void InteractiveContainer::layoutSizeChanged (int width, int height)
    {
        const float ratio = (float)width / (float)height;
        bool needResize = (ratio > ratio_*1.1f) || (ratio < ratio_*0.9f); // check for 10% deviation
        std::cout << "[INTERACTIVECONTAINER] Need resize: " + std::to_string(needResize) + " (" + std::to_string(ratio) + ")\n";
        if (needResize || !resized_)
        {
            std::cout << "[INTERACTIVECONTAINER] Layout has been changed: w" + std::to_string(width) + " h" + std::to_string(height) + "\n";

            // TODO wait for https://redmine.webtoolkit.eu/issues/8335
            const bool needRearange = ((ratio>=1.0f)^(ratio_>=1.0f));
            if (needRearange) rearrange(ratio);

            resized_ = true;
            ratio_ = ratio;
        }//if
    }

    void InteractiveContainer::rearrange(float ratio)
    {
        std::cout << "[INTERACTIVECONTAINER] Rearrange called for ratio: " + std::to_string(ratio) + "\n";

        // detach widgets
        auto chat = chatContainer_ ? static_cast<ChatContainer*>(layout_->removeWidget(chatContainer_).release()) : nullptr;
        auto options = optionsSection_ ? static_cast<OptionsContainer*>(layout_->removeWidget(optionsSection_).release()) : nullptr;

        // create or attach widgets
        if (ratio >= 1.0f)
            layout_ = setLayout(Wt::cpp14::make_unique<Wt::WHBoxLayout>());
        else
            layout_ = setLayout(Wt::cpp14::make_unique<Wt::WVBoxLayout>());

        chatContainer_ = layout_->addWidget(chat ? std::unique_ptr<ChatContainer>(chat) : Wt::cpp14::make_unique<ChatContainer>(), 1);
        optionsSection_ = layout_->addWidget(options ? std::unique_ptr<OptionsContainer>(options) : Wt::cpp14::make_unique<OptionsContainer>());
        setLayoutSizeAware(true);
    }

} /* namespace halfagame */
} /* namespace bsw */

